from os import listdir
from os import path
import logging
from filetraversalresult import FileTraversalResult

logger = logging.getLogger("filetraversal")

class FileTraversal:
    '''
    File traversal object that will operate on it's constructed targets.
    '''
    def __init__(self, foldermethod=None, filemethod=None):
        '''
        Constructor. "foldermethod" is the function called on each folder. "filemethod" is the function called on each file.
        '''
        
        if foldermethod is None and filemethod is None:
            logger.warning("No functions defined for folder or file traversal.")

        self.foldermethod = foldermethod
        self.filemethod = filemethod


    def recurse(self, rootpath, depth=0):
        '''
        Recursively traverses the "rootpath" folder, calling the FileTraversal object's "foldermethod" and "filemethod" methods
        as it goes. "depth" is an optional parameter that represents folder depth of the recursive operation.
        '''
        result = FileTraversalResult()
        folderlist = listdir(rootpath)

        #Simple recursive folder and file traversal.
        for entry in folderlist: 
            item = path.join(rootpath, entry)
            if path.isdir(item):
                recursive_result = self.recurse(item, depth + 1)
                result.maximizedepth(depth)
                result.maximizedepth(recursive_result.getmaximumdepth())
                if self.foldermethod is not None:
                    self.foldermethod(item)
                result.addfolders(item)
                result.addfolders(recursive_result.getfolderstraversed())
                result.addfiles(recursive_result.getfilestraversed())
            if path.isfile(item):
                if self.filemethod is not None:
                    self.filemethod(item)
                result.addfiles(item)

        result.maximizedepth(depth)

        #Finally, disable mutability of the result.
        if(depth == 0):
            result.disablemutability()
        return result



class FileTraversalResult:
    '''
    Holds result of file traversal.
    '''
    def __init__(self):
        super()
        self.folderstraversed = list()
        self.filestraversed = list()
        self.maximumfolderdepth = 0
        self.__ismutable = True
    
    def addfolders(self, folders):
        '''
        Appends traversed folders to the result list.
        '''
        if self.__ismutable:
            self.folderstraversed.append(folders)

    def addfiles(self, files):
        '''
        Appends traversed files to the result list.
        '''
        if self.__ismutable:
            self.filestraversed.append(files)

    def maximizedepth(self, depth):
        '''
        Sets maximum depth of traversal.
        '''
        if self.__ismutable:        
            if depth > self.maximumfolderdepth:
                self.maximumfolderdepth = depth

    def disablemutability(self):
        '''
        Sets the mutability if possible, will return the post-assignment mutability status (True/False).
        '''
        if self.__ismutable:
            self.__ismutable = False

    def getfolderstraversed(self):
        '''
        Returns folders traversed.
        '''
        return self.folderstraversed

    def getfilestraversed(self):
        '''
        Returns files traversed.
        '''
        return self.filestraversed

    def getmaximumdepth(self):
        '''
        Returns the maximum folder depth reached.
        '''
        return self.maximumfolderdepth


import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="file-traversal",
    version="0.1.0",
    author="Alex Beahm",
    author_email="alexanderbeahm@gmail.com",
    description="Simple injectible rescursive file system processor.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/AlexanderBeahm/file-traverser.git",
    packages=["file-traversal"],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)